close all
clc

%Globals
num_steps = 377+1;
test_file = 'F-16_18GHz_Demo_cfnprnumertime1.dat';
test_file2 = 'F-16_18GHz_Demo_cfnprnumertime2.dat';
test_file3 = 'F-16_18GHz_Demo_cfnprnumertime3.dat';
source_file = 'gauss.dat';
%Read in the data
[npr1, npr2] = readFile(test_file);
[npr3, npr4] = readFile(test_file2);
[npr5, npr6] = readFile(test_file3);
%Make FFT of a single point
[freq1, Ex1, Ey1, Ez1, timer, ExTD, EyTD, EzTD] = fft_single_point(npr1, num_steps);
[freq2, Ex2, Ey2, Ez2, timer2, ExTD2, EyTD2, EzTD2] = fft_single_point(npr3, num_steps);
[freq3, Ex3, Ey3, Ez3, timer3, ExTD3, EyTD3, EzTD3] = fft_single_point(npr5, num_steps);
%Read Source data
[gaussTime, gaussField] = readSource(source_file);
%FFT source
[gaussFreq, gaussFFTimag] = EMAfft(gaussTime, gaussField);
gaussFFT = abs(gaussFFTimag);
%Make an FFT of a frequency average
[maxTFx, minTFx, meanTFx] = Analyze_Data(npr1, num_steps, gaussFFT);
[maxTFx2, minTFx2, meanTFx2] = Analyze_Data(npr3, num_steps, gaussFFT);
[maxTFx3, minTFx3, meanTFx3] = Analyze_Data(npr5, num_steps, gaussFFT);

freqrange = [5e8 18e9];
dbrange = [-10 60];
figure
semilogx(freq1,maxTFx,'k',freq1,minTFx,'k',freq1,meanTFx,'r','LineWidth',2)
title('F-16 Canopy Shielding Effectiveness')
xlabel('Freqency (Hz)')
ylabel('SE (dB)')
set(gca,'FontSize',12)
set(gcf, 'Color', [1 1 1]);
xlim(freqrange)
legend('Upper bound simulation', 'Lower bound simulation', 'Mean simulation')

figure
semilogx(freq2,maxTFx2,'k',freq2,minTFx2,'k',freq2,meanTFx2,'r','LineWidth',2)
title('F-16 Avionics Bay Shielding Effectiveness')
xlabel('Freqency (Hz)')
ylabel('SE (dB)')
set(gca,'FontSize',12)
set(gcf, 'Color', [1 1 1]);
xlim(freqrange)
legend('Upper bound simulation', 'Lower bound simulation', 'Mean simulation')

figure
semilogx(freq3,maxTFx3,'k',freq3,minTFx3,'k',freq3,meanTFx3,'r','LineWidth',2)
title('F-16 Control Panel Shielding Effectiveness')
xlabel('Freqency (Hz)')
ylabel('SE (dB)')
set(gca,'FontSize',12)
set(gcf, 'Color', [1 1 1]);
xlim(freqrange)
legend('Upper bound simulation', 'Lower bound simulation', 'Mean simulation')


return

function [waveform,readin] = readFile(filename)
waveform = zeros(4725000,1);
disp('Reading file from disk: ')
disp(filename)
readin = dlmread(filename);
l=1;
for i = 1:length(readin)
    for k=1:9
        waveform(l)=readin(i,k);
        l = l+1;
    end
end
end

function [x,y] = read(test_file)
winsize = 20;
data1 = dlmread(test_file);
x = data1(:,1);
y = data1(:,2);
end

function [frequency1, functFFT] = EMAfft(time,funct)
% T = time(2) - time(1);
% Fs = 1/T;
% L = 2*length(time);
% NFFT = round(2^(round(log2(L))));
% Y = fft(funct,NFFT)/L;
% frequency1 = Fs/2*(1:(NFFT/2));
% frequency1 = reshape(frequency1,[],1);
% functFFT = Y(1:(NFFT/2));


T = time(2) - time(1);
Fs = 1/T;
L = length(funct);
t = (0:L-1)*T;
Y = fft(funct);
P2 = abs(Y/L);
P1 = P2(1:L/2+1);
P1(2:end-1) = 2*P1(2:end-1);
frequency1 = Fs*(0:(L/2))/L;
functFFT = P1;
end

function [freq, Ex, Ey, Ez, timer, ExTD, EyTD, EzTD] = fft_single_point(npr, num_steps)

timer = npr(1:num_steps:end);
ExTD = npr(2:num_steps:end);
EyTD = npr(3:num_steps:end);
EzTD = npr(4:num_steps:end);
[freq ,Ex] = EMAfft(timer, ExTD);
[freq ,Ey] = EMAfft(timer, EyTD);
[freq ,Ez] = EMAfft(timer, EzTD);
end

function [wavebwa] = bwa(freq,wave,npct)
%Filtering
df = freq(2) - freq(1);
nFmax = length(wave);
wavebwa = zeros(nFmax,1);
for i = 1:1:nFmax
    
    valf = freq(i) * npct/200;
    npc = round(valf/df) +1;
    nst = i - npc;
    nnd = i + npc;
    if nst < 1
        nst =1;
    end
    if nnd > nFmax
        break
    end
    if nnd > nst
        wavebwa(i) = mean(wave(nst:nnd));
    end
end
end

function [time, field] = readSource(filename)
data = dlmread(filename);
time = data(:,1);
field = data(:,2);
end

function [maxTFx, minTFx, meanTFx] = Analyze_Data(npr, num_steps, gaussFFT)
timer = npr(1:num_steps:end);
ExTD = npr(2:num_steps:end);
[freq, sumExComp] = EMAfft(timer, ExTD);
ExBWA = bwa(freq, abs(sumExComp), 5);
maxTFx = -20 * log10(ExBWA ./ gaussFFT);
minTFx = -20 * log10(ExBWA ./ gaussFFT);
sumTFx = -20 * log10(ExBWA ./ gaussFFT);
for i = 1:1:124
    fprintf('Processing Point # %s of %s \n', num2str(i+1), num2str(125))
    k = 3*i +2;
    ExTD = npr(k:num_steps:end);
    [freq, Ex] = EMAfft(timer, ExTD);
    ExBWA = bwa(freq, abs(Ex), 5);
    TFx = -20 * log10(ExBWA ./ gaussFFT);
    
%     figure
%     semilogx(freq,TFx,'k','LineWidth',2)
%     title('F-16 Canopy Shielding Effectiveness')
%     xlabel('Freqency (Hz)')
%     ylabel('SE (dB)')
%     set(gca,'FontSize',12)
%     set(gcf, 'Color', [1 1 1]);
    
    maxTFx = max(TFx, maxTFx);
    minTFx = min(TFx, minTFx);
    sumTFx = sumTFx + TFx;
end
meanTFx = sumTFx/125;
end
